/**
 * $Id: MemoryGuard.java,v 1.5 2009/01/20 13:51:34 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 19.01.2009 by oliver (ob@oasd.de)
 */
package patterntesting.runtime.monitor;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

import patterntesting.runtime.annotation.RunBackground;
import patterntesting.runtime.util.Converter;

/**
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 19.01.2009
 * @version $Revision: 1.5 $
 */
public class MemoryGuard {
    
    private static final Log log = LogFactoryImpl.getLog(MemoryGuard.class);
    private static final long maxMem = Runtime.getRuntime().maxMemory();
    
    public static long getFreeMemory() {
        Runtime runtime = Runtime.getRuntime();
        long totalMem = runtime.totalMemory();
        return runtime.freeMemory() + (maxMem - totalMem);
    }
    
    public static int getFreeMemoryInPercent() {
        long freeMem = getFreeMemory();
        int rate = (int) ((freeMem + 50) * 100 / maxMem);
        return rate;
    }
    
    public static String getFreeMemoryAsString() {
        return Converter.getMemoryAsString(getFreeMemory());
    }
    
    /**
     * @see #logMemory(Log)
     */
    public static void logMemory() {
        logMemory(log);
    }
    
    /**
     * Logs a message if free memory falls down below x% of maximal heap size
     * where x is:
     * <ul>
     *  <li>        x <  1%: logs a fatal message</li>
     *  <li>  1% <= x <  2%: logs an error</li>
     *  <li>  1% <= x < 10%: logs a warning</li>
     *  <li> 10% <= x < 20%: logs an info message</li>
     *  <li> 20% <= x < 50%: logs debug message</li>
     *  <li>   x >= 50%:     tracing</li>
     * </ul>
     */
    public static void logMemory(final Log log) {
        int freeMemRate = getFreeMemoryInPercent();
        if (freeMemRate < 10) {
            System.gc();
            if (log.isTraceEnabled()) {
                log.trace("gc() called because free memory is below 10% ("
                        + freeMemRate + "%)");
            }
            freeMemRate = getFreeMemoryInPercent();
        }
        if (freeMemRate < 1) {
            if (log.isFatalEnabled()) {
                log.fatal(getMemoryLogMessage(freeMemRate));
            }
            
        } if (freeMemRate < 2) {
            if (log.isErrorEnabled()) {
                log.error(getMemoryLogMessage(freeMemRate));
            }
        } else if (freeMemRate < 10) {
            if (log.isWarnEnabled()) {
                log.warn(getMemoryLogMessage(freeMemRate));
            }
        } else if (freeMemRate < 20) {
            if (log.isInfoEnabled()) {
                log.info(getMemoryLogMessage(freeMemRate));
            }
        } else if (freeMemRate < 50) {
            if (log.isDebugEnabled()) {
                log.debug(getMemoryLogMessage(freeMemRate));
            }
        } else {
            if (log.isTraceEnabled()) {
                log.trace(getMemoryLogMessage(freeMemRate));
            }
        }
    }
    
    public static String getMemoryLogMessage() {
        int freeMemRate = getFreeMemoryInPercent();
        return getMemoryLogMessage(freeMemRate);
    }
    
    private static String getMemoryLogMessage(int freeMemRate) {
        return freeMemRate + "% of memory is free (" + getFreeMemoryAsString()
                + ")";
    }
    
    /**
     * Every x milliseconds the free memory is checked and printed into the log
     * (according the log level).
     * 
     * @param interval (the sleep intervall in milliseconds, e.g. 300000 for 5 min.)
     * @throws InterruptedException
     */
    @RunBackground
    public static void logFreeMemory(long interval) throws InterruptedException {
        while (true) {
            logMemory();
            Thread.sleep(interval);
        }
    }

}

/**
 * $Log: MemoryGuard.java,v $
 * Revision 1.5  2009/01/20 13:51:34  oboehm
 * getMemoryLogMessage() added
 *
 * Revision 1.4  2009/01/20 13:38:21  oboehm
 * logMemory(..) provides now different parameters
 *
 * Revision 1.3  2009/01/20 09:08:52  oboehm
 * parameter added to logMemory(..)
 *
 * Revision 1.2  2009/01/19 16:03:43  oboehm
 * MemoryGuard added to watch memory
 *
 * Revision 1.1  2009/01/19 15:42:58  oboehm
 * MemoryGuard added (to be able to watch memory consumption)
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-rt/src/main/java/patterntesting/runtime/monitor/MemoryGuard.java,v $
 */
